#ifndef MapObject_h
#define MapObject_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include "MapPass.h"

#include <Math/TDRectangle.h>
#include <Transactions/TransactionObject.h>

class wxDC;
class MapObjectContainer;
class MapEditor;
class MapSelection;
class MapSelectionManager;

class MapObject;

WX_DECLARE_LIST(MapObject,MapObjectList);

/**
 * \brief A MapObject represents an object in a map.
 *
 * See \ref map_objects for more information on implementing custom MapObject
 * classes.
 *
 * \note A refernce to a MapObject does not guarantee that its container stays
 * alive. If you need the container, ensure that you reference the container
 * all the time while you hold the reference to the MapObject.
 */
class MapObject: public TransactionObject {
        
        DECLARE_CLASS(MapObject)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(MapObject)
        
        friend class MapSelectionActionsHandler;
        friend class MapObjectContainer;
        
public:
        /**
         * \brief Create a new object.
         *
         * \param transactionManager    The TransactionManager to attach this object to
         */
	MapObject(TransactionManager* transactionManager);
        
        /**
         * \brief Destruct this MapObject.
         *
         * May not be called manually (only via DeRef())
         */
        virtual ~MapObject();
        
        /**
         * \brief Paint this object.
         *
         * The given editor provides projection and mapping information
         * required to paint the object. Note that an object can be
         * shown in several independent editors.
         *
         * \param dc            The device context to paint on.
         * \param editor        The editor to paint in.
         * \param pass          The pass to paint.
         */
        virtual void Paint(wxDC& dc, MapEditor* editor, const MapPass& pass)=0;
        
        /**
         * \brief Check if this object intersects with a given rectangle.
         *
         * \param editor        The editor providing the context of the rectangle.
         * \param r             The hit-rectangle in logical coordinates of the
         *                      editor.
         *
         * \return true if and only if this objects intersects the given
         *         rectangle.
         */
        virtual bool Hit(MapEditor* editor, TDRectangle r)=0;
        
        /**
         * \brief Create a selection for this object.
         *
         * \param manager       The selection manager to attach the selection to.
         *
         * \return the new MapSelection object.
         *
         * See \ref selections for more info on implementing custom selection
         * classes.
         */
        virtual MapSelection* CreateSelection(MapSelectionManager* manager)=0;
        
        /**
         * \brief Return the device bounds of this object in the given editor.
         *
         * This should return a bounding box that exactly contains the area
         * overwritten when painting the object in OnPaint().
         */
        virtual wxRect GetDeviceBounds(MapEditor* editor)=0;
        
        /**
         * \brief Return the logical bounds of this object in the given editor.
         */
        virtual TDRectangle GetLocalBounds(MapEditor* editor);
        
        /**
         * \brief Return the geographic bounds of this object in the given editor.
         */
        virtual TDRectangle GetGeographicBounds()=0;
        
        /**
         * \brief Schedule this object for repaint.
         */
        void Damage();
        
        /**
         * \brief Remove this object from its container.
         *
         * The object is not actually deleted, but may linger in a transaction
         * waiting for the removal to be potentially undone until the transaction
         * dies.
         */
        void RemoveFromContainer();
        
        /**
         * \brief Inform this object that it is about to be deattached from its container.
         */
        virtual void OnDetach() {}
        
        /**
         * \brief Get the container of this object.
         */
        MapObjectContainer* GetContainer() const { return _container; }
        
        /**
         * \brief The standard map pass.
         */
        static const MapPass StandardPass;
protected:
        MapObjectContainer* _container;
        
        /**
         * \brief Set the container of this object.
         */
        void SetContainer(MapObjectContainer* container) {
                BeforeWrite();
                _container=container;
                AfterWrite();
        }
        
        /**
         * \brief Trigger all necessary actions before a change to the object.
         *
         * These actions are:
         *
         * -# Call TransactionObject::BeforeWrite() to trigger storage of a memento.
         * -# Call MapObject::Damage() to trigger redraw of the formerly covered
         *    area.
         */
        virtual void BeforeWrite() {
                TransactionObject::BeforeWrite();
                Damage(); // force redraw of region formerly covered
        }
        
        /**
         * \brief Trigger all necessary actions after a change to the object.
         *
         * These actions are:
         *
         * -# Call TransactionObject::AfterWrite() to trigger listeners
         * -# Call MapObject::Damage() to trigger redraw of the changed object.
         */
        virtual void AfterWrite() {
                Damage(); // force redraw of changed object
                TransactionObject::AfterWrite();
        }

        class Memento: public TransactionObject::Memento {
                friend class MapObject;
        public:
                Memento(const MapObject& object):
                        TransactionObject::Memento(object) {
                        _container=object._container;
                }
                
                virtual ~Memento() {
                }
        protected:
                MapObjectContainer *_container;
        };
public:
        virtual Memento* CreateMemento() {
                return new Memento(*this);
        }
        
        virtual void RestoreMemento(wxObject* object);
};
#endif
