#ifndef MapObjectContainer_h
#define MapObjectContainer_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include "MapLayer.h"
#include "MapObject.h"

/**
 * \brief A MapObjectContainer is a MapLayer actually containing MapObject instances.
 */
class MapObjectContainer: public MapLayer {
        
        DECLARE_CLASS(MapObjectContainer)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(MapObjectContainer)
        
        friend class MapObject;
public:
	MapObjectContainer(TransactionManager* tmgr, wxString name);
        virtual ~MapObjectContainer();
        
        virtual void Paint(wxDC& dc, MapEditor* editor, const MapPass& pass);
        
        virtual void GetHitObjects(MapObjectList& list,
                                   MapEditor* editor,
                                   TDRectangle r,
                                   bool includeIntersected=true);
        
        virtual wxRect GetDeviceBounds(MapEditor* editor);
        
        virtual TDRectangle GetLocalBounds(MapEditor* editor);
        
        virtual TDRectangle GetGeographicBounds();
        
        virtual void GetObjectsOfClass(MapObjectList& list, wxClassInfo* targetClass);
        
        /**
         * \brief Attach a MapObject to this layer on top.
         */
        virtual void AttachObject(MapObject* obj);
        
        /**
         * \brief Attach a MapObject to this layer at the given index.
         *
         * \param index   The index to place the object at, counting from zero
         *                starting on the bottom.
         */
        virtual void AttachObject(MapObject* obj, size_t index);
        
        /**
         * \brief Detach a MapObject from this layer.
         */
        virtual void DetachObject(MapObject* obj);
        
        /**
         * \brief Get the count of objects in this layer.
         */
        size_t GetObjectCount() const {
                return _objects.size();
        }
        
        /**
         * \brief Get the index of the given object in this layer.
         *
         * \return -1 if the object was not found in this layer,
         *         the index, starting at zero for the bottom, of the object
         *         otherwise.
         */
        int GetObjectIndex(MapObject* obj) const {
                const int index=_objects.IndexOf(obj);
                if (index==wxNOT_FOUND)
                        return -1;
                else
                        return index;
        }
        
        /**
         * \brief Move the given MapObject to the top of the layer.
         */
        void MoveObjectToTop(MapObject* obj);
        
        /**
         * \brief Move the given MapObject up in the object stack of the layer.
         */
        void MoveObjectUp(MapObject* obj);

        /**
         * \brief Move the given MapObject down in the object stack of the layer.
         */
        void MoveObjectDown(MapObject* obj);

        /**
         * \brief Move the given MapObject to the bottom of the layer.
         */
        void MoveObjectToBottom(MapObject* obj);
        
        /**
         * \brief Flip the order of all objects in this layer.
         */
        void FlipObjectsOrder();
protected:
        class Memento: public MapLayer::Memento {
                friend class MapObjectContainer;
        public:
                Memento(const MapObjectContainer& container):
                        MapLayer::Memento(container) {
                        _objects=container._objects;
                        MapObjectList::Node *node;
                        for (node=_objects.GetFirst();node;node=node->GetNext()) {
                                node->GetData()->TransactionObject::Ref();
                        }
                }
                
                virtual ~Memento() {
                        MapObjectList::Node *node;
                        for (node=_objects.GetFirst();node;node=node->GetNext()) {
                                node->GetData()->TransactionObject::UnRef();
                        }
                }
        protected:
                MapObjectList _objects;
        };
public:
        virtual Memento* CreateMemento() {
                return new Memento(*this);
        }
        
        virtual void RestoreMemento(wxObject* object);
protected:
        
        /**
         * \brief Selectively schedule a MapObject for repaint in all MapEditor
         * instances displaying this layer.
         */
        void Damage(MapObject* obj);
        
        MapObjectList _objects;
};

#endif

