#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "MapEditor.h"
#include "ModeManager.h"
#include "Mode.h"
#include <iostream>

#include <wx/listimpl.cpp>
WX_DEFINE_LIST(ModeList);

IMPLEMENT_CLASS(ModeManager,wxEvtHandler);

ModeManager::ModeManager(MapEditor* editor):
        _statusField(0),
        _editor(editor) {
}

ModeManager::~ModeManager() {
        RemoveAll(true);
}

wxStatusBar* ModeManager::GetStatusBar() {
        wxWindow* window=GetEditor()->GetDrawingWindow();
        wxWindow* topWnd = ::wxGetTopLevelParent(window);
        
        if ( topWnd )
        {
                wxFrame* pFrame = wxDynamicCast(topWnd,wxFrame);
                if ( pFrame )
                {
                        wxStatusBar* pStatusBar = pFrame->GetStatusBar();
                        return pStatusBar;
                }
        }
        
        return NULL;
}

void ModeManager::UpdateStatus() {
        wxStatusBar* statusBar=GetStatusBar();
	if (!statusBar)
		return;
	ModeList::Node *first=_modes.GetFirst();
	if (!first)
		return;
	Mode* mode=first->GetData();
	statusBar->SetStatusText(mode->GetStatus(),_statusField);
}

/* Insert the given mode on top of the stack */
void ModeManager::InsertOnTop(Mode* mode) {
        mode->FireBeforeEnteringMode();
        _modes.Insert(mode);
        
        SetupMode(mode);
}

/* Insert the given mode above the given other mode */
void ModeManager::InsertAbove(Mode* mode, Mode* above) {
        ModeList::Node* aboveNode=_modes.Find(above);
        wxASSERT(aboveNode!=NULL);
        mode->FireBeforeEnteringMode();
        _modes.Insert(aboveNode,mode);
        
        SetupMode(mode);
}

/* Insert the given mode below the given other mode */
void ModeManager::InsertBelow(Mode* mode, Mode* below) {
        ModeList::Node* belowNode=_modes.Find(below);
        wxASSERT(belowNode!=NULL);
        belowNode=belowNode->GetNext();
        mode->FireBeforeEnteringMode();
        if (belowNode!=NULL)
                _modes.Insert(belowNode,mode);
        else
                _modes.Append(mode);
        
        SetupMode(mode);
}

/* Insert the given mode at the bottom of the stack */
void ModeManager::InsertAtBottom(Mode* mode) {
        mode->FireBeforeEnteringMode();
        _modes.Append(mode);
        
        SetupMode(mode);
}

/* Remove the topmost mode */
bool ModeManager::RemoveTopmost(bool force) {
        ModeList::Node* topNode=_modes.GetFirst();
        if (topNode==NULL)
                return false;
        return Remove(topNode->GetData(),force);
}

/* Remove the given mode */
bool ModeManager::Remove(Mode* mode, bool force) {
        ModeList::Node* modeNode=_modes.Find(mode);
        if (modeNode==NULL)
                return false;
        
        if (!force && !mode->CanExit())
                return false;
        
        mode->Leave(force);
        
        mode->SetModeManager(NULL);
        bool ret=_modes.DeleteNode(modeNode);
        
        mode->FireAfterLeavingMode();
        
        delete mode;
        
        UpdateStatus();
        
        return ret;
}

/* Remove all modes from the top */
void ModeManager::RemoveAll(bool force) {
        while (RemoveTopmost(force));
}

bool ModeManager::RemoveAt(Mode* startMode, bool force) {
        if (!_modes.Find(startMode))
                return false;
        bool foundStartMode=false;
        while (_modes.GetCount()>0 && !foundStartMode) {
                foundStartMode=(_modes.GetFirst()->GetData()==startMode);
                if (!RemoveTopmost(force))
                        return false;
        }
        return true;
}

bool ModeManager::ProcessEvent(wxEvent& evt) {
        ModeList::Node* modeNode;
        bool handled=false;
        
        for (modeNode=_modes.GetFirst();!handled && modeNode;modeNode=modeNode->GetNext())
	{
                handled=modeNode->GetData()->ProcessEvent(evt);
        }
		
	if (evt.GetEventType()==wxEVT_LEFT_DOWN) {
		// pass this on to allow focus handling
		handled=false;
	}
        
        return handled || wxEvtHandler::ProcessEvent(evt);
}

/* Initialise the mode properly */
void ModeManager::SetupMode(Mode* mode) {
        mode->SetModeManager(this);

        if (!mode->Enter()) {
                /*
                 * mode vetoed being entered, remove it forcibly.
                 *
                 * Note that the mode could have pushed other modes
                 * instead so we cannot assumed that it is the topmost
                 * mode anymore. Therefore use Remove() instead of
                 * RemoveTopmost().
                 */
                Remove(mode,true);
                return;
        }
        
        /* Update the status bar, if any */
        UpdateStatus();
}

